#
# Copyright (C) 2023 Juan Domingo (Juan.Domingo@uv.es)
#
# This script is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

FillGene <- function ()
{
 d <- select(org.Hs.eg.db,keys=keys(org.Hs.eg.db),columns=c("ENTREZID","SYMBOL"))
 sink("FillGene.sql")
 for(i in 1:nrow(d))
 {
    row <- d[i,]
    # YES, there is at least one moronic genetist that used a single-quote character in the symbol name of a gene...
    # See https://xkcd.com/327/
    if (str_length(row[2])>0) 
    {
       if (grepl("'",row[2]))
       {
        wk<-paste(unlist(row[2]))
        h<-strsplit(wk,"'")
        cat(paste0("INSERT INTO Gene VALUES('",row[1],"','",h[[1]][1],"\\'",h[[1]][2],"');\n"))
       }
       else 
       {
         cat(paste0("INSERT INTO Gene VALUES('",row[1],"','",row[2],"');\n"))
       } 
   }
   else
   {
     cat(paste0("INSERT INTO Gene VALUES('",row[1],"','",row[2],"');\n"))
   } 
 }
# This is to accomodate our special Entrez value to be associated to intergenic variations
# This must be an integer value not used neither now nor in the future for Entrez identifiers. We have chosen -1.
 cat("INSERT INTO Gene VALUES('-1','Intergenic');\n")
 sink()
}

FillGeneInEnsemble <- function()
{
 d <- select(org.Hs.eg.db,keys=keys(org.Hs.eg.db),columns=c("ENTREZID","ENSEMBL"))
 l<-unique(d[,2])
 sink("FillGeneInEnsemble.sql")
 for (ens in l)
 {
  if (!isNA(ens))
  {
   cat(paste0("INSERT INTO GeneInEnsemble VALUES('",ens,"');\n"))
  }
 }
 # This is to accomodate our special Ensemble value to be associated to intergenic variations
 # This must be an integer value not used neither now nor in the future for Ensemble identifiers. We have chosen Intergenic
 cat("INSERT INTO GeneInEnsemble VALUES('Intergenic');\n")
 sink()
}

Fillhas_ensembleid <- function()
{
 d <- select(org.Hs.eg.db,keys=keys(org.Hs.eg.db),columns=c("ENTREZID","ENSEMBL"))
 sink("Fillhas_ensenmbleid.sql")
 for(i in 1:nrow(d)) 
 {
    row <- d[i,]
    if (!isNA(row[2]))
    {
       cat(paste0("INSERT INTO has_ensembleid VALUES('",row[1],"','",row[2],"');\n"))
    }
 }
 # This is to accomodate our special value to be associated to intergenic variations
 cat("INSERT INTO has_ensembleid VALUES('-1','Intergenic');\n")
 sink();
}

FillGeneTranscript <- function()
{
 d <- select(org.Hs.eg.db,keys=keys(org.Hs.eg.db),columns=c("ENTREZID","ENSEMBLTRANS"))
 l<-unique(d[,2])
 sink("FillGeneTranscript.sql")
 for (tra in l)
 {
  if (!isNA(tra))
  {
   cat(paste0("INSERT INTO GeneTranscript VALUES('",tra,"');\n"))
  }
 }
 # This is to accomodate our special value to be associated to intergenic variations
 cat("INSERT INTO GeneTranscript VALUES('Intergenic');\n")
 sink()
}

Filltranscribes_as <- function()
{
 d <- select(org.Hs.eg.db,keys=keys(org.Hs.eg.db),columns=c("ENTREZID","ENSEMBLTRANS"))
 sink("Filltranscribes_as.sql")
 for(i in 1:nrow(d)) 
 {
    row <- d[i,]
    if (!isNA(row[2]))
    {
       cat(paste0("INSERT INTO transcribes_as VALUES('",row[1],"','",row[2],"');\n"))
    }
 }
 cat("INSERT INTO transcribes_as VALUES('-1','Intergenic');\n")
 sink();
}

FillSynonim <- function()
{
 d <- select(org.Hs.eg.db,keys=keys(org.Hs.eg.db),columns=c("ENTREZID","ALIAS"))
 l<-unique(d[,2])
 sink("FillSynonym.sql")
 for (syn in l)
 {
  where=which(syn==d[,2]);
  if (length(where)==1)
  {
   ent=d[where,1];
  }
  else
  {
   ent=d[where[1],1];
  }
  if (grepl("'",syn))
  {
   wk<-paste(unlist(syn))
   h<-strsplit(wk,"'")
   cat(paste0("INSERT INTO Synonym VALUES('",h[[1]][1],"\\'",h[[1]][2],"','",ent,"');\n"))
  }
  else
  {
   cat(paste0("INSERT INTO Synonym VALUES('",syn,"','",ent,"');\n"))
  }
 }
 # This is to accomodate our special value to be associated to intergenic variations
 cat("INSERT INTO Synonym VALUES('Intergenic','-1');\n")
 sink()
}

